@@ -301,6 +301,10 @@ When the parameter type is not annotated, the parameter type is inferred from th
 
 When this setting is `false`, the type of the parameter is `any` when it is not annotated.
 ]]
+config.type.checkTableShape              = -- TODO: need translate!
+[[
+对表的形状进行严格检查。
+]]
 config.doc.privateName                   = -- TODO: need translate!
 'Treat specific field names as private, e.g. `m_*` means `XXX.m_id` and `XXX.m_type` are private, witch can only be accessed in the class where the definition is located.'
 config.doc.protectedName                 = -- TODO: need translate!
